"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RubyGemsDatasource = void 0;
const tslib_1 = require("tslib");
const marshal_1 = require("@qnighy/marshal");
const logger_1 = require("../../../logger");
const result_1 = require("../../../util/result");
const url_1 = require("../../../util/url");
const rubyVersioning = tslib_1.__importStar(require("../../versioning/ruby"));
const datasource_1 = require("../datasource");
const common_1 = require("./common");
const http_1 = require("./http");
const metadata_cache_1 = require("./metadata-cache");
const schema_1 = require("./schema");
const versions_endpoint_cache_1 = require("./versions-endpoint-cache");
class RubyGemsDatasource extends datasource_1.Datasource {
    static id = 'rubygems';
    metadataCache;
    constructor() {
        super(RubyGemsDatasource.id);
        this.http = new http_1.RubygemsHttp(RubyGemsDatasource.id);
        this.versionsEndpointCache = new versions_endpoint_cache_1.VersionsEndpointCache(this.http);
        this.metadataCache = new metadata_cache_1.MetadataCache(this.http);
    }
    defaultRegistryUrls = ['https://rubygems.org'];
    defaultVersioning = rubyVersioning.id;
    registryStrategy = 'hunt';
    versionsEndpointCache;
    async getReleases({ packageName, registryUrl, }) {
        // istanbul ignore if
        if (!registryUrl) {
            return null;
        }
        const registryHostname = (0, url_1.parseUrl)(registryUrl)?.hostname;
        let result;
        if (registryHostname === 'rubygems.org') {
            result = result_1.Result.wrap(this.versionsEndpointCache.getVersions(registryUrl, packageName)).transform((versions) => this.metadataCache.getRelease(registryUrl, packageName, versions));
        }
        else if (registryHostname === 'rubygems.pkg.github.com' ||
            registryHostname === 'gitlab.com') {
            result = this.getReleasesViaDeprecatedAPI(registryUrl, packageName);
        }
        else {
            result = (0, common_1.getV1Releases)(this.http, registryUrl, packageName).catch(() => this.getReleasesViaDeprecatedAPI(registryUrl, packageName));
        }
        const { val, err } = await result.unwrap();
        if (val) {
            return val;
        }
        if (err instanceof Error) {
            this.handleGenericErrors(err);
        }
        logger_1.logger.debug({ packageName, registryUrl }, `Rubygems fetch error: ${err}`);
        return null;
    }
    getReleasesViaDeprecatedAPI(registryUrl, packageName) {
        const path = (0, url_1.joinUrlParts)(registryUrl, `/api/v1/dependencies`);
        const query = (0, url_1.getQueryString)({ gems: packageName });
        const url = `${path}?${query}`;
        const bufPromise = this.http.getBuffer(url);
        return result_1.Result.wrap(bufPromise).transform(({ body }) => {
            const data = marshal_1.Marshal.parse(body);
            const releases = schema_1.MarshalledVersionInfo.safeParse(data);
            return releases.success
                ? result_1.Result.ok({ releases: releases.data })
                : result_1.Result.err(releases.error);
        });
    }
}
exports.RubyGemsDatasource = RubyGemsDatasource;
//# sourceMappingURL=index.js.map